import "dart:io";

import "package:directed_graph/directed_graph.dart";
import "package:multipack/discovery.dart";
import "package:multipack/graph.dart";
import "package:pubspec/pubspec.dart";

class FsGraphBuilder extends GraphBuilder {
  @override
  Future<DirectedGraph<Package>> buildDependencyGraph() async {
    final root = Directory.current;

    final vertices = await findPackages(root)
        .map((package) => Vertex<Package>(package))
        .toList();

    return DirectedGraph<Package>(
      {
        for (var vertex in vertices)
          vertex: vertex.data.pubspec.allDependencies.keys
              .map(
                (dep) => vertices.firstWhere(
                  (v) => v.data.pubspec.name == dep,
                  orElse: () => null,
                ),
              )
              .where((v) => v != null)
              .toList(),
      },
      comparator: (
        Vertex<Package> vertex1,
        Vertex<Package> vertex2,
      ) =>
          vertex1.data.pubspec.name.compareTo(vertex2.data.pubspec.name),
    );
  }
}

Stream<Package> findPackages(Directory root) =>
    findPackageDirectories(root).asyncMap(
      (dir) async => Package(
        path: dir.path,
        pubspec: await PubSpec.load(dir),
      ),
    );

Stream<Directory> findPackageDirectories(Directory root) => root
    .list(recursive: true)
    .where(
      (entity) =>
          entity is File &&
          entity.path.endsWith(Platform.pathSeparator + "pubspec.yaml"),
    )
    .cast<File>()
    .map(
      (pubspec) => pubspec.parent.absolute,
    );
